      SUBROUTINE AKAIKE(X,N,A0,B0,IP,IQ,NIT,EPS,PER,A,B,SIG2,IFLAG) 
C 
C   This program implements the Akaike MLE algorithm for 
C   ARMA parameter estimation of real data.  A complete  
C   description of the method is contained in Section 10.3.
C 
C   Input Parameters: 
C 
C     X     - Real array of dimension Nx1 of real data points 
C     N     - Number of real data points 
C     A0    - Real array of dimension IPx1 containing initial 
C             estimates of AR parameters for Newton-Raphson 
C             iteration arranged as A0(1) to A0(IP)
C     B0    - Real array of dimension IQx1 containing initial 
C             minimum-phase estimates of MA parameters for Newton-
C             Raphson iteration arranged as B0(1) to B0(IQ)
C     IP    - AR model order (must be one or greater) 
C     IQ    - MA model order (must be one or greater) 
C     NIT   - Maximum number of iterations allowed  
C     EPS   - Small positive number to test for ill-conditioning
C             of Hessian
C     PER   - Stopping criterion, if percentage change from previous 
C             iteration in value of function to be minimized is  
C             less than PER, iteration is halted 
C 
C   Output Parameters: 
C 
C     A     - Real array of dimension IPx1 of AR parameter 
C             estimates arranged as A(1) to A(IP)
C     B     - Real array of dimension IQx1 of MA parameter 
C             estimates arranged as B(1) to B(IQ)
C     SIG2  - Excitation white noise variance estimate
C     IFLAG - This is a flag indicating conditions of termination 
C             of iteration: 
C               =-1, if one of the di's in CHOLESKY is less than
C                    EPS causing premature termination (Hessian
C                    is ill-conditioned)
C               = 0, percentage error change in function  
C                    less than PER causing termination 
C               = 1, MA filter estimate not minimum-phase 
C                    causing premature termination 
C               = 2, all NIT iterations completed causing
C                    termination             
C 
C   External Subroutines:
C
C     CHOLESKY - see Appendix 2B
C     STEPDOWN - see Appendix 6C
C    
C   Notes: 
C
C     The calling program must dimension arrays X,A0,
C     B0,A,B.  The arrays BVC,HC,BC,BBC,Y,Y1,Z,RY,RZ,
C     RYZ,RZY,G,H,BV,P must be dimensioned greater than
C     or equal to the variable dimensions shown.  The arrays
C     Y,E in VAR must be dimensioned greater than or equal
C     to N and the array H in HESS, BVECT must be dimensioned
C     to agree with the array H in AKAIKE.  The arrays XL,Y,D
C     in CHOLESKY must be dimensioned to be greater than or
C     equal to (IP+IQ)x(IP+IQ), IP+IQ, and IP+IQ, respectively.
C     The array AA in STEPDOWN must be dimensioned to agree with
C     BBC in AKAIKE.
C
C   Verification Test Case:
C
C     If the real test data listed in Appendix 1.A is input
C     as X, with N=32, IP=7, IQ=3, NIT=100, EPS=1.0E-15, PER=0.1,
C     and with the initial estimates set as
C       A0(1)  =  0.22197
C       A0(2)  = -1.09315   
C       A0(3)  = -0.59794    
C       A0(4)  =  0.62299    
C       A0(5)  =  0.28311    
C       A0(6)  =  0.13978    
C       A0(7)  =  0.13255    
C       B0(1)  =  3.57056E-02
C       B0(2)  = -0.41063    
C       B0(3)  =  6.39064E-02    
C     then the output should be
C       A(1)  =  -0.38280
C       A(2)  =  -0.85412    
C       A(3)  =   0.25719    
C       A(4)  =   0.59831    
C       A(5)  =  -0.43314    
C       A(6)  =   9.61728E-02
C       A(7)  =   0.21083    
C       B(1)  =  -0.44578    
C       B(2)  =  -0.24197    
C       B(3)  =   0.46824    
C       SIG2  =   1.67210
C       IFLAG = 0
C     for a DEC VAX 11/780.
C  
      COMPLEX BVC(IP+IQ),HC((IP+IQ)*(IP+IQ+1)/2),BC(IP+IQ),BBC(IQ,IQ) 
      DIMENSION A(1),B(1),A0(1),B0(1),X(1),Y(N),Y1(N), 
     * Z(N),RY(IP+1),RZ(IQ+1),RYZ(IP),RZY(IQ),G(IP+IQ),
     * H(IP+IQ,IP+IQ),BV(IP+IQ),P(IQ) 
C   Compute estimate of white noise variance or Q(a,b) using initial
C   ARMA filter parameter estimates (10.1)
      CALL VAR(X,N,IP,IQ,A0,B0,SIG2P) 
      IFLAG=3 
      DO 10 I=1,IP 
10    A(I)=A0(I) 
      DO 20 I=1,IQ 
20    B(I)=B0(I) 
      DO 100 K=1,NIT 
C   Filter data with 1/B(z) to produce y[n] sequence
C   and with A(z)/B^2(z) to produce z[n] sequence (10.5)
      CALL POLEFILT(X,N,IQ,B,Y) 
      CALL POLEFILT(Y,N,IQ,B,Y1) 
      CALL ZEROFILT(Y1,N,IP,A,Z) 
C   Compute estimates of autocorrelations and cross-
C   correlations, ryy[n],rzz[n],ryz[n],rzy[n]
      CALL CORRCOMP(N,Y,Z,IP,IQ,RY,RZ,RYZ,RZY) 
C   Compute gradient vector (10.3)
      CALL GRAD(A,B,IP,IQ,RY,RZ,G) 
C   Compute Hessian matrix (10.4)
      CALL HESS(IP,IQ,RY,RZ,RYZ,RZY,H) 
C   Compute right-hand-side vector of linear equations
C   to be solved for next iterate (10.6)
      CALL BVECT(A,B,IP,IQ,G,H,BV) 
C   Set right-hand-side vector and Hessian equal to 
C   complex quantities for use in CHOLESKY
      L=0 
      DO 40 J=1,IP+IQ 
      BVC(J)=CMPLX(BV(J),0.) 
      DO 30 I=1,J 
      L=L+1 
30    HC(L)=CMPLX(H(I,J),0.) 
40    CONTINUE 
C   Solve equations to produce next iterate (10.6)
      M=IP+IQ 
      CALL CHOLESKY(HC,BVC,M,EPS,IFLAG) 
C   Terminate program if equations are ill-conditioned
      IF(IFLAG.EQ.-1)RETURN 
      IFLAG=3
C   Check to see if estimate of B(z) is minimum-phase by
C   checking the reflection coefficients
      DO 50 I=1,IQ 
50    BC(I)=BVC(IP+I) 
      IF(IQ.EQ.1)BBC(1,1)=BC(1) 
      PMA=1.
      IF(IQ.GT.1)CALL STEPDOWN(IQ,BC,PMA,BBC,P0,P) 
C   If B(z) estimate is not minimum-phase, terminate program
      DO 60 I=1,IQ 
      IF(CABS(BBC(I,I)).LT.1.0)GO TO 60 
      IFLAG=1 
      RETURN 
60    CONTINUE 
C   Set up new estimates of ARMA parameters
      DO 70 I=1,IP 
70    A(I)=REAL(BVC(I)) 
      DO 80 I=1,IQ 
80    B(I)=REAL(BVC(I+IP)) 
C   Compute new estimate of white noise variance or Q(a,b) and
C   if the change in the variance (error function to be 
C   minimized) is less than PER, terminate program (10.1)
      CALL CHKDN(SIG2P,N,X,A,B,IP,IQ,PER,SIG2,IFLAG) 
      IF(IFLAG.EQ.0)RETURN
C   Check to see if all iterations completed and if so
C   terminate program
      IF(K.LT.NIT)GO TO 90 
      IFLAG=2 
      RETURN 
90    SIG2P=SIG2 
100   CONTINUE 
      RETURN 
      END 
      SUBROUTINE VAR(X,N,IP,IQ,A,B,SIG2)
C   This program computes the estimate of the white
C   noise variance, which is the error function to
C   be minimized.
      DIMENSION X(1),Y(N),E(N),A(1),B(1) 
      CALL POLEFILT(X,N,IQ,B,Y) 
      CALL ZEROFILT(Y,N,IP,A,E) 
      SUM=0. 
      DO 10 I=1,N 
10    SUM=SUM+E(I)*E(I) 
      SIG2=SUM/N 
      RETURN 
      END 
      SUBROUTINE POLEFILT(X,N,IQ,B,Y) 
C   This program implements an all-pole filter, 1/B(z), with
C   the initial conditions of the filter set equal to zero.
      DIMENSION X(1),Y(1),B(1) 
      Y(1)=X(1) 
      IF(IQ.EQ.1)GO TO 25 
      DO 20 I=2,IQ 
      Y(I)=X(I) 
      DO 10 J=1,I-1 
10    Y(I)=Y(I)-B(J)*Y(I-J) 
20    CONTINUE 
25    DO 40 I=IQ+1,N 
      Y(I)=X(I) 
      DO 30 J=1,IQ 
30    Y(I)=Y(I)-B(J)*Y(I-J) 
40    CONTINUE 
      RETURN 
      END 
      SUBROUTINE ZEROFILT(X,N,IP,A,Y) 
C   This program implements an FIR filter with coefficients
C   given by A.  The values of the input X(I) for I < 1 are
C   set equal to zero. 
      DIMENSION X(1),Y(1),A(1) 
      Y(1)=X(1) 
      IF(IP.EQ.1)GO TO 25 
      DO 20 I=2,IP 
      Y(I)=X(I) 
      DO 10 J=1,I-1 
10    Y(I)=Y(I)+A(J)*X(I-J) 
20    CONTINUE 
25    DO 40 I=IP+1,N 
      Y(I)=X(I) 
      DO 30 J=1,IP 
30    Y(I)=Y(I)+A(J)*X(I-J) 
40    CONTINUE 
      RETURN 
      END 
      SUBROUTINE CORRCOMP(N,Y,Z,IP,IQ,RY,RZ,RYZ,RZY) 
C   This program computes the autocorrelations and cross-
C   correlations needed in the Akaike algorithm.
      DIMENSION Y(1),Z(1),RY(1),RZ(1),RYZ(1),RZY(1)
      DO 10 K=0,IP 
      CALL CORR(N,K,Y,Y,R) 
10    RY(K+1)=R 
      DO 20 K=0,IQ 
      CALL CORR(N,K,Z,Z,R) 
20    RZ(K+1)=R 
      DO 30 K=0,IP-1 
      CALL CORR(N,K,Y,Z,R) 
30    RYZ(K+1)=R 
      DO 40 K=1,IQ-1 
      CALL CORR(N,K,Z,Y,R) 
40    RZY(K+1)=R 
      RETURN 
      END 
      SUBROUTINE CORR(N,LAG,X,Y,R) 
C   This program computes a cross (auto) - correlation
C   from two input data sequences.
      DIMENSION X(1),Y(1) 
      SUM=0. 
      DO 10 J=1,N-LAG 
10    SUM=SUM+X(J)*Y(J+LAG) 
      R=SUM/N 
      RETURN 
      END 
      SUBROUTINE GRAD(A,B,IP,IQ,RY,RZ,G) 
C   This program computes the gradient vector
C   needed in the Akaike algorithm.
      DIMENSION A(1),B(1),RY(1),RZ(1),G(1) 
      DO 20 K=1,IP 
      G(K)=2.*RY(K+1) 
      DO 10 I=1,IP 
10    G(K)=G(K)+2.*A(I)*RY(IABS(K-I)+1) 
20    CONTINUE 
      DO 40 L=1,IQ 
      G(IP+L)=-2.*RZ(L+1) 
      DO 30 I=1,IQ 
30    G(IP+L)=G(IP+L)-2.*B(I)*RZ(IABS(L-I)+1) 
40    CONTINUE 
      RETURN 
      END 
      SUBROUTINE HESS(IP,IQ,RY,RZ,RYZ,RZY,H) 
C   This program computes the Hessian needed in the
C   Akaike algorithm.
      DIMENSION RY(1),RZ(1),RYZ(1),RZY(1),H(IP+IQ,IP+IQ) 
      DO 20 J=1,IP 
      DO 10 K=1,IP 
10    H(J,K)=2.*RY(IABS(J-K)+1) 
20    CONTINUE 
      DO 40 J=IP+1,IP+IQ 
      DO 30 K=IP+1,IP+IQ 
30    H(J,K)=2.*RZ(IABS(J-K)+1) 
40    CONTINUE 
      DO 60 J=1,IP 
      DO 50 K=IP+1,IP+IQ 
      IF(J.GE.(K-IP))H(J,K)=-2.*RYZ(J-(K-IP)+1) 
50    IF(J.LT.(K-IP))H(J,K)=-2.*RZY((K-IP)-J+1) 
60    CONTINUE 
      DO 80 J=IP+1,IP+IQ 
      DO 70 K=1,IP 
70    H(J,K)=H(K,J) 
80    CONTINUE 
      RETURN 
      END 
      SUBROUTINE BVECT(A,B,IP,IQ,G,H,BV) 
C   This program computes the right-hand-side vector
C   of the linear equations to be solved for next iterate.
      DIMENSION A(1),B(1),G(1),H(IP+IQ,IP+IQ),BV(1) 
      DO 20 I=1,IP+IQ 
      BV(I)=-G(I) 
      DO 10 J=1,IP+IQ 
      IF(J.LE.IP)T=A(J) 
      IF(J.GT.IP)T=B(J-IP) 
10    BV(I)=BV(I)+H(I,J)*T 
20    CONTINUE 
      RETURN 
      END 
      SUBROUTINE CHKDN(SIG2P,N,X,A,B,IP,IQ,PER,SIG2,IFLAG) 
C   This program computes the estimate of the white noise
C   variance (error function to be minimized) given the
C   ARMA filter parameter estimates.
      DIMENSION X(1),A(1),B(1) 
      CALL VAR(X,N,IP,IQ,A,B,SIG2) 
      ERR=100.*(SIG2-SIG2P)/SIG2P
      IF(ABS(ERR).LT.PER)IFLAG=0
      RETURN
      END


