      SUBROUTINE ARMAPSD(IP,IQ,A,B,SIG2,NEXP,PSD)
C
C   This subroutine computes a set of PSD values across the
C   frequency band [-1/2,1/2), given the parameters of a complex or real
C   ARMA model. The FFT is used to evaluate the numerator
C   and denominator polynomials of the ARMA PSD function. PSD(I) corre-
C   sponds to the PSD at frequency F=-1/2+(I-1)/L, where L=2**NEXP is
C   the number of frequency samples desired. 
C
C   The PSD for a pure AR model may be obtained as a special case 
C   by setting IQ to zero. Similarly, the PSD for a pure
C   MA model may be obtained as a special case by setting IP to zero.
C
C   For a real model set the imaginary parts of the ARMA filter
C   parameters equal to zero.
C
C   Input parameters:
C
C     IP   - AR model order (for MA process IP=0)
C     IQ   - MA model order (for AR process IQ=0)
C     A    - Complex array of dimension IPx1 of AR parameters
C            arranged as A(1) to A(IP)
C     B    - Complex array of dimension IQx1 of MA parameters 
C            arranged as B(1) to B(IQ)
C     SIG2 - Variance of excitation noise
C     NEXP - Power of two which determines number of frequency
C            samples desired, L=2**NEXP (L must be greater than
C            or equal to the maximum of IP+2 and IQ+2.)
C
C   Output parameters:
C
C     PSD  - Real array of dimension 2**NEXPx1 of PSD values
C
C   External subroutines:
C
C     FFT    - see Appendix 2A
C
C
C   Notes: 
C 
C     The calling program must dimension arrays PSD and A,B even
C     if IP=0 or IQ=0.  The arrays DEN,XNUM,P must be dimensioned 
C     to be greater than or equal to the variable dimensions shown.
C     The array Y in FFT must be dimensioned greater than or equal
C     to 2**NEXP.
C
C   Verification Test Case:
C
C     If IP=2, IQ=2, A(1)=(-0.0707,-1.2), A(2)=(-0.72,0.),
C     B(1)=(0.,-0.707), B(2)=(-0.25,0.), SIG2=2., NEXP=9,
C     then the output should be
C       PSD(100) = 0.90657
C       PSD(200) = 0.97848
C       PSD(300) = 6.04913
C       PSD(400) = 2.27597
C       PSD(500) = 1.64754
C     for a DEC VAX 11/780.
C
      COMPLEX A(1),B(1)
      COMPLEX DEN(2**NEXP),XNUM(2**NEXP)
      DIMENSION P(2**NEXP),PSD(1)
      L=2**NEXP
C   PSD computation begins
5     IF(IP.EQ.0)GO TO 30
C   Compute denominator frequency function
      DEN(1)=(1.,0.)
      DO 10 I=1,IP
10    DEN(I+1)=A(I)
      DO 20 I=IP+2,L
20    DEN(I)=(0.,0.)
      INVRS=1
      CALL FFT(DEN,NEXP,INVRS)
30    IF(IQ.EQ.0)GO TO 60
C   Compute numerator frequency function
      XNUM(1)=(1.,0.)
      DO 40 I=1,IQ
40    XNUM(I+1)=B(I)
      DO 50 I=IQ+2,L
50    XNUM(I)=(0.,0.)
      INVRS=1
      CALL FFT(XNUM,NEXP,INVRS)
C   Compute PSD values
60    IF(IP.EQ.0)GO TO 80
      IF(IQ.EQ.0)GO TO 100
      DO 70 I=1,L
70    P(I)=SIG2*(REAL(XNUM(I))**2+AIMAG(XNUM(I))**2)/
     * (REAL(DEN(I))**2+AIMAG(DEN(I))**2)
      GO TO 120
80    DO 90 I=1,L
90    P(I)=SIG2*(REAL(XNUM(I))**2+AIMAG(XNUM(I))**2)
      GO TO 120
100   DO 110 I=1,L
110   P(I)=SIG2/(REAL(DEN(I))**2+AIMAG(DEN(I))**2)
C   Transpose halves of FFT outputs so that first PSD 
C   value is at a frequency of -1/2
120   DO 130 I=1,L/2
      PSD(I+L/2)=P(I)
130   PSD(I)=P(I+L/2)     
      RETURN
      END

