      SUBROUTINE BLACKTUKEY(X,N,MODE,W,M,NEXP,PBT) 
C   This program computes the Blackman-Tukey spectral estimator
C   as given by (4.23).  Either the biased or unbiased 
C   autocorrelation estimator may be used as well as a lag window.
C   The spectral estimate is evaluated at the frequencies 
C   F=-1/2+(I-1)/L for I=1,2,...L.  The number of frequencies is 
C   given by L=2**NEXP.
C
C   Input Parameters:
C
C     X    - Complex array of dimension Nx1 of data points
C     N    - Number of data points
C     MODE - Set equal to zero for unbiased autocorrelation
C            estimator; otherwise, biased estimator used
C     W    - Real array of dimension 2M+1 of lag window weights;
C            W(1),...,W(M+1),...,W(2M+1) correspond
C            to w[-M],...,w[0],...,w[M].
C     M    - Largest lag desired
C     NEXP - Power of two which determines number of frequency
C            samples desired, L=2**NEXP; must be chosen so that
C            L is greater than or equal to 2*M+2
C
C   Output Parameters:
C
C     PBT  - Real array of dimension L=2**NEXPx1 of samples of
C            the Blackman-Tukey spectral estimate, where PBT(I)
C            corresponds to the spectral estimate at the
C            frequency F=-1/2+(I-1)/L 
C
C   External Subroutines:
C
C     FFT         - see Appendix 2A
C     CORRELATION - see Appendix 4D
C
C   Notes: 
C
C     The calling program must dimension the arrays X,W,PBT.
C     The arrays R,RCORR,P must be dimensioned greater than
C     or equal to the variable dimension shown.  The array Y
C     in FFT must be dimensioned greater than or equal to 2**NEXP.
C
C   Verification Test Case:
C
C     If the complex data test case listed in Appendix 1.A is
C     input as X, with N=32, MODE=1, W(I)=1.-ABS(I-11.)/10.
C     for I=1,2,...,21, M=10, and NEXP=9, then the output should be
C       PBT(100) =  1.54635
C       PBT(200) =  4.55061
C       PBT(300) = 12.39030
C       PBT(400) =  1.47874
C       PBT(500) =  4.95506
C     for a DEC VAX 11/780.
C
      COMPLEX X(1),R(2**NEXP),RCORR(M+1) 
      DIMENSION W(1),P(2**NEXP),PBT(1) 
      PI=4.*ATAN(1.) 
C   Compute the autocorrelation estimates from the data
      M1=M+1 
      CALL CORRELATION(N,M1,MODE,X,X,RCORR) 
C   Window the M+1 autocorrelation estimates and insert them
C   into last M+1 locations of RCORR. Then, fill first
C   M points of RCORR array with the complex conjugate of
C   the last M points (shift autocorrelation sequence to
C   the right by M samples so that FFT may be used)
      R(M1)=W(M1)*RCORR(1) 
      DO 10 I=1,M 
      R(M1+I)=W(M1+I)*RCORR(I+1) 
10    R(M1-I)=W(M1-I)*CONJG(RCORR(I+1)) 
C   Zero pad array of windowed autocorrelation samples to
C   obtain an array of dimension equal to L
      L=2**NEXP
      DO 20 I=2*M+2,L
20    R(I)=(0.,0.) 
C   Compute FFT of autocorrelation sequence
      INVRS=1 
      CALL FFT(R,NEXP,INVRS)
C   Compensate for shifting the autocorrelation sequence to the
C   right by M samples
      DO 30 I=1,L
      F=(I-1.)/L
      ARG=2.*PI*F*M
30    P(I)=REAL(R(I)*CEXP(CMPLX(0.,ARG)))
C   Transpose halves of FFT output so that first PSD sample is 
C   at a frequency of -1/2
      DO 40 I=1,L/2
      PBT(I+L/2)=P(I)
40    PBT(I)=P(I+L/2)
      RETURN
      END

