      SUBROUTINE CHOLESKY(A,B,N,EPS,IFLAG)
C   This program solves a hermitian positive definite set of 
C   complex linear simultaneous equations using the Cholesky 
C   decomposition method. See (2.53)-(2.55).
C
C              AX=B
C
C   Input Parameters:
C
C     N     - Order of the matrix (number of linear equations)
C     EPS   - Small positive number to test for ill-
C             conditioning of matrix A
C     A     - Complex array of dimension N*(N+1)/2 of matrix 
C             elements stored columnwise for upper triangular part,
C             i.e., A(1,1) stored as A(1), A(1,2) stored as A(2), 
C             A(2,2) stored as A(3),etc. (The lower triangular part 
C             is found by hermitian symmetry.)
C     B     - Complex array of dimension Nx1 of right-hand-side 
C             vector elements
C
C   Output Parameters:
C
C     IFLAG - Ill-conditioning indicator
C               =  0 for normal program termination
C               = -1 if one of the di's is less than EPS
C                    causing premature termination
C     B     - Complex array of dimension Nx1 containing solution 
C             X vector stored in place of B on output
C   
C   Notes: 
C
C     The calling program must dimension arrays A,B. The arrays
C     XL,Y,D must be dimensioned greater than or equal to the variable
C     dimensions shown.
C 
C   Verification Test Case:
C
C     If A(1)=(2.,0.), A(2)=(-1.,-1.), A(3)=(2.,0.), B(1)=(1.,-1),
C     B(2)=(0.,0.), N=2, and EPS=1.0E-15, then the output should be
C       B(1)=(0.99999,-0.99999)
C       B(2)=(0.00000,-0.99999)
C       IFLAG=0
C     for a DEC VAX 11/780.
C
      COMPLEX A(1),B(1),XL(N,N),Y(N)
      DIMENSION D(N)
C   Factor into triangular and diagonal form (2.55)
      IFLAG=0
      L=1
      D(1)=REAL(A(L))
      DO 40 I=2,N
      DO 20 J=1,I-1
      L=L+1
      XL(I,J)=CONJG(A(L))/D(J)
      IF(J.EQ.1)GO TO 20
      DO 10 K=1,J-1
10    XL(I,J)=XL(I,J)-XL(I,K)*CONJG(XL(J,K))*D(K)/D(J)
20    CONTINUE
      L=L+1
      D(I)=REAL(A(L))
      DO 30 K=1,I-1
30    D(I)=D(I)-D(K)*CABS(XL(I,K))**2
C   Test for non-positive value of di
      IF(D(I).GT.EPS)GO TO 40
      IFLAG=-1
      RETURN
40    CONTINUE
C   Solve for intermediate column vector solution (2.53)
      Y(1)=B(1)
      DO 60 K=2,N
      Y(K)=B(K)
      DO 50 J=1,K-1
50    Y(K)=Y(K)-XL(K,J)*Y(J)
60    CONTINUE
C   Solve for final column vector solution (2.54)
      B(N)=Y(N)/D(N)
      DO 80 K=N-1,1,-1
      B(K)=Y(K)/D(K)
      DO 70 J=K+1,N
70    B(K)=B(K)-CONJG(XL(J,K))*B(J)
80    CONTINUE
      RETURN
      END












