      SUBROUTINE COVMCOV(X,N,IP,MODE,EPS,IFLAG,A,SIG2)
C   This program implements either the covariance method
C   (7.8) and (7.9) or the modified covariance method 
C   (7.22)-(7.24) for estimation of the AR parameters.
C
C   Input Parameters:
C
C     X     - Complex array of dimension Nx1 of data points
C     N     - Number of data points
C     IP    - AR model order desired
C     MODE  - Set =1 for modified covariance method, otherwise
C             covariance method implemented
C     EPS   - Small positive number to test for ill-conditioning
C             of matrix, required in CHOLESKY used to solve linear
C             equations
C
C   Output Parameters:
C
C     IFLAG - Ill-conditioning indicator
C               =  0 for normal program termination
C               = -1 if one of the di's in Cholesky decomposition
C                    is less than EPS causing premature termination
C     A     - Complex array of dimension IPx1 of AR filter 
C             parameter estimates arranged as A(1) to A(IP)
C     SIG2  - Excitation white noise variance estimate
C
C   External Subroutines:
C
C     CHOLESKY - see Appendix 2B
C
C   Notes: 
C
C     The calling program must dimension arrays X,A.  
C     The CC array must be dimensioned greater than or 
C     equal to the variable dimensions shown.  The arrays
C     XL,Y,D in CHOLESKY must be dimensioned greater than
C     or equal to IPxIP, IP, and IP, respectively.
C
C   Verification Test Case:
C
C     If the complex test data listed in Appendix 1.A is input
C     as X, with N=32, IP=10, MODE=0, and EPS=1.0E-15, then the 
C     output should be
C       IFLAG=0
C       A(1) = (-0.22160,-1.87537E-02)
C       A(2) = (-1.08574,-4.21279E-02)
C       A(3) = ( 0.14958, 4.17785E-03)
C       A(4) = ( 0.83520, 0.14371    )
C       A(5) = (-0.69723,-0.22038    )
C       A(6) = ( 0.27110,-0.21128    )
C       A(7) = ( 0.59138, 0.21102    )
C       A(8) = (-0.21453, 0.22058    )    
C       A(9) = (-0.38712,-0.14536    )
C       A(10)= ( 0.33533,-8.34980E-02)
C       SIG2 = 0.11476
C     for a DEC VAX 11/780.
C     If the complex test data listed in Appendix 1.A is input
C     as X, with N=32, IP=10, MODE=1, and EPS=1.0E-15, then the 
C     output should be
C       IFLAG=0
C       A(1) = (-0.10112, 0.12298    )
C       A(2) = (-1.07629,-6.32359E-02)
C       A(3) = (-0.14683,-8.97811E-02)
C       A(4) = ( 0.86527, 9.84762E-02)
C       A(5) = (-0.61774,-7.41049E-02)
C       A(6) = ( 0.38049,-0.32719    )
C       A(7) = ( 0.63081, 0.22249    )
C       A(8) = (-0.21936, 0.34745    )    
C       A(9) = (-0.52164,-7.32829E-02)
C       A(10)= ( 0.38135,-0.19113    )
C       SIG2 = 0.16222
C     for a DEC VAX 11/780.
C
      COMPLEX X(1),A(1),CC(IP*(IP+1)/2),C,SUM
C   Compute autocorrelation estimates and insert them in
C   matrix (stored in symmetric mode in CC)
      L=1
      DO 50 K=1,IP
      DO 40 J=1,K
      CC(L)=(0.,0.)
C   For covariance method use only single 
C   autocorrelation term (7.8)
      DO 10 I=IP+1,N
10    CC(L)=CC(L)+CONJG(X(I-J))*X(I-K)
C   For modified covariance method add additional 
C   autocorrelation term (7.22)
      IF(MODE.NE.1)GO TO 30
      DO 20 I=1,N-IP
20    CC(L)=CC(L)+X(I+J)*CONJG(X(I+K))
30    L=L+1
40    CONTINUE
50    CONTINUE
      DO 80 J=1,IP
C   Compute right hand vector of covariance equations
      A(J)=(0.,0.)
C   For covariance method use only single autocorrelation
C   term (7.8)
      DO 60 I=IP+1,N
60    A(J)=A(J)-CONJG(X(I-J))*X(I)
C   For modified covariance method add additional
C   autocorrelation term (7.23)
      IF(MODE.NE.1)GO TO 80
      DO 70 I=1,N-IP
70    A(J)=A(J)-X(I+J)*CONJG(X(I))
80    CONTINUE
C   Compute AR filter parameter estimate
      CALL CHOLESKY(CC,A,IP,EPS,IFLAG)
      IF(IFLAG.EQ.-1)RETURN
C   Compute estimate of white noise variance
      SUM=(0.,0.)
      DO 120 KK=1,IP+1
      C=(0.,0.)
      K=KK-1
C   For covariance method use only single autocorrelation
C   term (7.9)
      DO 90 I=IP+1,N
90    C=C+CONJG(X(I))*X(I-K)
C   For modified covariance method add additional
C   autocorrelation term (7.24)
      IF(MODE.NE.1)GO TO 110
      DO 100 I=1,N-IP
100   C=C+X(I)*CONJG(X(I+K))
110   IF(K.EQ.0)SUM=SUM+C
      IF(K.NE.0)SUM=SUM+C*A(K)
120   CONTINUE
C   Complete estimate of white noise variance 
      IF(MODE.EQ.1)SIG2=REAL(SUM)/(2.*(N-IP))
      IF(MODE.NE.1)SIG2=REAL(SUM)/(N-IP)
      RETURN
      END

