      SUBROUTINE GENDATA(IP,IQ,A,B,SIG2,N,V,X)
C
C   This program generates a complex AR, MA, or ARMA time series 
C   given the filter parameters, excitation noise variance, and 
C   a complex array of zero mean, unit variance uncorrelated 
C   random variables (complex white noise).  For an AR or ARMA 
C   process the starting transient is eliminated because the 
C   initial conditions of the filter are specified to place the 
C   filter output in statistical steady state.  
C   
C   For a real AR, MA, or ARMA time series set the imaginary parts
C   of the filter parameters (complex arrays A,B) equal to zero
C   as well as the imaginary part of the white noise input (complex
C   array V).  The real part of the samples in the V array should now
C   have a variance of one.  On output take the real part of the 
C   complex array X.
C
C   Input Parameters:
C
C     IP   - AR model order (for MA process IP=0)
C     IQ   - MA model order (for AR process IQ=0)
C     A    - Complex array of dimension IPx1 of AR filter parameters
C            arranged as A(1) to A(IP) 
C     B    - Complex array of dimension IQx1 of MA filter parameters
C            arranged as B(1) to B(IQ)
C     SIG2 - Variance of excitation noise
C     N    - Number of data points desired
C     V    - Complex array of dimension (N+IQ)x1 of white noise 
C            samples; each complex noise sample should have real (Vr[k])
C            and imaginary (Vi[k]) parts with zero means and variances of 
C            1/2.  The random variables {Vr[1],Vi[1],...,Vr[N+IQ],Vi[N+IQ]} 
C            should all be uncorrelated with each other.  For a 
C            Gaussian time series each Vr[k] and Vi[k] should be 
C            Gaussian. Program WGN may be used to generate Gaussian
C            Vr[n],Vi[n] samples.  For a real time series let Vi[n]=0 
C            and set the variance of each Vr[n] equal to one.            
C
C   Output Parameters:
C  
C     X    - Complex array of dimension Nx1 of time series samples
C
C   External Subroutines:
C
C     STEPDOWN - see Appendix 6C
C  
C   Notes: 
C 
C     The calling program must dimension arrays V,X and also 
C     A,B even if IP=0 or IQ=0.  The arrays X1,AA,RHO must be 
C     dimensioned greater than or equal to the variable dimensions 
C     shown.  If IP=0, set the dimensions of AA and RHO as AA(1,1) 
C     and RHO(1).  The array AA in STEPDOWN must be dimensioned 
C     greater than or equal to (IP,IP) if IP>0 or to (1,1) if IP=0.
C     The user must insure that the AR filter parameters chosen
C     result in a stable all-pole filter.
C
C   Verification Test Case:
C
C     If IP=2, IQ=2, A(1)=(-0.0707,-1.2), A(2)=(-0.72,0.),
C     B(1)=(0.,-0.707), B(2)=(-0.25,0.), SIG2=2., N=5, and 
C     V(I)=(I,I) for I=1,2,...,7, then the output should be
C       X(1)=(2.43056,7.50579), X(2)=(1.79000,8.79596),
C       X(3)=(1.33120,10.18507), X(4)=(1.23115,10.7224), 
C       X(5)=(2.30939,11.70134)
C     for a DEC VAX 11/780.
C       
      COMPLEX A(1),B(1),V(1),X(1),X1(N+IQ),AA(IP,IP)
      DIMENSION RHO(IP)
      IF(IP.NE.0)GO TO 20
      DO 10 K=1,N+IQ
10    X1(K)=SQRT(SIG2)*V(K)
      GO TO 80
C   Set initial conditions for AR filtering
20    IF(IP.GT.1)CALL STEPDOWN(IP,A,SIG2,AA,RHO0,RHO)
      IF(IP.EQ.1)RHO0=SIG2/(1.-CABS(A(1))**2)
      X1(1)=SQRT(RHO0)*V(1)
      IF(IP.EQ.1)GO TO 50
      DO 40 K=2,IP
      X1(K)=SQRT(RHO(K-1))*V(K)
      DO 30 L=1,K-1
30    X1(K)=X1(K)-AA(L,K-1)*X1(K-L)
40    CONTINUE
C   Generate AR part of time series
50    DO 70 K=IP+1,N+IQ
      X1(K)=SQRT(SIG2)*V(K)
      DO 60 L=1,IP
60    X1(K)=X1(K)-A(L)*X1(K-L)
70    CONTINUE
80    IF(IQ.NE.0)GO TO 100
      DO 90 K=1,N
90    X(K)=X1(K)
      RETURN
C   Generate MA part of time series
100   DO 120 K=1,N
      X(K)=X1(K+IQ)
      DO 110 L=1,IQ
110   X(K)=X(K)+B(L)*X1(K+IQ-L)
120   CONTINUE
      RETURN
      END


