      SUBROUTINE LEVINSON(R,IP,A,AA,RHO)
C
C   This program implements the Levinson recursion.
C   See (6.46)-(6.48) and Figure 6.4.
C
C   Input Parameters:
C
C     R   - Complex array of dimension (IP+1)x1 of autocorrelation 
C           samples where R(1),R(2),...,R(IP+1) correspond to 
C           rxx[0],rxx[1],...,rxx[p]
C     IP  - Order or dimension of Yule-Walker equations to be
C           solved
C
C   Output Parameters:
C
C     A   - Complex array of dimension IPx1 containing solution,
C           arranged as A(1) to A(IP)
C     AA  - Complex array of dimension IPxIP containing all lower
C           order solutions as well as desired one; AA(I,J)
C           corresponds to the Ith coefficient for the Jth order
C           solution (AA(I,IP)=A(I))
C     RHO - Real array of dimension IPx1 containing prediction
C           error powers where RHO(J) is the prediction error
C           power for the Jth order predictor (RHO(IP)=white noise 
C           variance)
C
C   Notes:  
C
C     The calling program must dimension the arrays R,A,AA,RHO.
C     The array AA must be dimensioned in the calling program
C     to be greater than or equal to the variable dimensions
C     shown and in this program to agree with calling program 
C     dimensions.
C
C   Verification Test Case:
C
C     If R(1)=(2.,0.), R(2)=(-1.,1.), R(3)=(0.,0.), and IP=2, then
C     the output should be
C       A(1)=AA(1,2)=(0.99999,-0.99999), A(2)=AA(2,2)=(0.00000,-0.99999),
C       AA(1,1)=(0.50000,-0.50000), RHO(1)=1.00000, RHO(2)=2.38418E-07
C     for a DEC VAX 11/780.
C
      COMPLEX R(1),AA(IP,IP),A(1),B
      DIMENSION RHO(1)
C   Initialization of Levinson recursion  
      RHO0=R(1)
      AA(1,1)=-R(2)/R(1)
      RHO(1)=(1.-CABS(AA(1,1))**2)*R(1)
      A(1)=AA(1,1)
      IF(IP.EQ.1)GO TO 50
C   Begin recursion by computing reflection coefficient (6.46)
      DO 30 I=2,IP
      B=-R(I+1)
      DO 10 K=1,I-1
10    B=B-AA(K,I-1)*R(I+1-K)
      AA(I,I)=B/RHO(I-1)
C   Update in order the prediction error filter and prediction
C   error power, (6.47) and (6.48)
      DO 20 K=1,I-1
20    AA(K,I)=AA(K,I-1)+AA(I,I)*CONJG(AA(I-K,I-1))
      RHO(I)=(1.-CABS(AA(I,I))**2)*RHO(I-1)
30    CONTINUE
C   Store final solution of Yule-Walker equations
      DO 40 I=1,IP
40    A(I)=AA(I,IP)
50    RETURN
      END

