      SUBROUTINE LSMYWE(X,N,IP,IQ,M,EPS,A,B,SIG2,IFLAG)
C   This program implements the least squares modified
C   Yule-Walker equation estimator (10.18) for the AR 
C   parameters of an ARMA process.  The unbiased autocorre-
C   lation estimator is used.  The MA parameters and  
C   the white noise variance are estimated by filtering the 
C   data by an estimate of A(z) and applying Durbin's method
C   to the data at the output of the filter (see Section 8.4).
C   The large order AR model in Durbin's method is chosen
C   to have an order of N/5.
C
C   Input Parameters:
C
C     X     - Complex array of dimension Nx1 of data points
C     N     - Number of complex data points
C     IP    - Desired AR model order (must be one or greater)
C     IQ    - Desired MA model order (must be one or greater)
C     M     - (M-IQ) is the number of equations used in LSMYWE
C             estimator
C     EPS   - Small positive number to test for ill-conditioning
C             of matrix, required in CHOLESKY used to solve
C             linear equations
C
C   Output Parameters:
C
C     A     - Complex array of dimension IPx1 of AR parameter
C             estimates arranged as A(1) to A(IP)
C     B     - Complex array of dimension IQx1 of MA parameter
C             estimates arranged as B(1) to B(IQ)
C     SIG2  - Excitation white noise variance estimate
C     IFLAG - Ill-conditioning indicator for solution of least
C             squares equations
C               =  0 for normal termination
C               = -1 if one of the di's in Cholesky decomposition
C                    is less than EPS causing premature termination
C
C   External Subroutines:
C
C     CHOLESKY    - see Appendix 2B
C     CORRELATION - see Appendix 4D
C     LEVINSON    - see Appendix 6B
C     AUTOCORR    - see Appendix 7B
C     DURBIN      - see Appendix 8A
C
C   Notes: 
C
C     The calling program must dimension arrays X,A,B.
C     The arrays R,RR,AA,Y must be dimensioned greater
C     than or equal to the variable dimensions shown.
C     The arrays XL,Y,D in CHOLESKY must be dimensioned
C     greater than or equal to IPxIP, IP, and IP, respectively.
C     The arrays A,AP in DURBIN must be dimensioned greater
C     than or equal to [N/5] and [N/5]+1, respectively. ([M]
C     denotes the largest integer less than or equal to M).
C     The arrays R,AA,RHO in AUTOCORR must be dimensioned
C     greater than or equal to [N/5]+1,[N/5]x[N/5], and [N/5],
C     respectively.  The array AA in LEVINSON must be dimensioned
C     to agree with the dimensions of AA in AUTOCORR.
C
C   Verification Test Case:
C   
C     If the complex test data listed in Appendix 1.A is input
C     as X, with N=32, IP=7, IQ=3, M=15, and EPS=1.0E-15, then 
C     the output should be
C       A(1) = ( 0.37106    ,-0.10668    )
C       A(2) = (-1.09528    ,-0.35745    )
C       A(3) = (-0.78537    , 9.54137E-02)
C       A(4) = ( 0.54507    , 0.47929    )
C       A(5) = ( 0.41105    , 8.75429E-02)
C       A(6) = ( 0.18216    ,-0.33953    )
C       A(7) = ( 0.13420    ,-0.10521    )
C       B(1) = ( 0.15233    ,-0.10394    )    
C       B(2) = (-0.43829    ,-0.26486    )
C       B(3) = (-4.66616E-02, 1.92027E-02)
C       SIG2 = 1.52461 
C       IFLAG=0
C     for a DEC VAX 11/780.
C
      COMPLEX X(1),A(1),B(1),R(M+1),RR(M-IQ,IP),AA(IP*(IP+1)/2),Y(N)
C   Compute unbiased autocorrelation estimates
      LAG=M+1
      CALL CORRELATION(N,LAG,0,X,X,R)
C   Fill in the (M-IQ)xIP autocorrelation matrix used
C   in the least squares solution
      DO 20 I=1,M-IQ
      DO 10 J=1,IP
      IF(IQ+I-J.GE.0)RR(I,J)=R(IQ+1+I-J)
      IF(IQ+I-J.LT.0)RR(I,J)=CONJG(R(J-I-IQ+1))
10    CONTINUE
20    CONTINUE
C   Compute R^H R matrix and store in symmetric mode (upper
C   triangular part only) (10.18)
      L=1
      DO 50 J=1,IP
      DO 40 I=1,J
      AA(L)=(0.,0.)
      DO 30 K=1,M-IQ
30    AA(L)=AA(L)+CONJG(RR(K,I))*RR(K,J)
      L=L+1
40    CONTINUE
50    CONTINUE
C   Compute -R^H r or right-hand-side vector (10.18)
      DO 70 I=1,IP
      A(I)=(0.,0.)
      DO 60 K=1,M-IQ
60    A(I)=A(I)-CONJG(RR(K,I))*R(IQ+1+K)
70    CONTINUE
C   Solve least squares equations (10.18)
      CALL CHOLESKY(AA,A,IP,EPS,IFLAG)
C   Terminate program if equations to be solved are ill-conditioned
      IF(IFLAG.EQ.-1)RETURN
C   Filter data with estimated A(z) to yield
C   approximate MA time series
      CALL CZEROFILT(X,N,IP,A,Y)
C   Apply Durbin's algorithm to estimate MA filter
C   parameters and white noise variance
      LL=N/5
      CALL DURBIN(Y,N,IQ,LL,B,SIG2)
      RETURN
      END
      SUBROUTINE CZEROFILT(X,N,IP,A,Y) 
C   This program implements an FIR filter with coefficients
C   given by A.  The values of the input X(I) for I < 1 are
C   set equal to zero.
      COMPLEX X(1),Y(1),A(1)           
      Y(1)=X(1) 
      IF(IP.EQ.1)GO TO 25 
      DO 20 I=2,IP 
      Y(I)=X(I) 
      DO 10 J=1,I-1 
10    Y(I)=Y(I)+A(J)*X(I-J) 
20    CONTINUE 
25    DO 40 I=IP+1,N 
      Y(I)=X(I) 
      DO 30 J=1,IP 
30    Y(I)=Y(I)+A(J)*X(I-J)
40    CONTINUE
      RETURN
      END

