      SUBROUTINE MAYNEFIR(X,N,IP,IQ,L,EPS,NEXP,A,B,SIG2,IFLAG) 
C   This program implements the Mayne-Firoozan method for the  
C   estimation of the parameters of a complex ARMA process
C   (10.23)-(10.27).
C 
C   Input Parameters: 
C 
C     X     - Complex vector with first N locations containing
C             data points (see Notes below)
C     N     - Number of complex data points 
C     IP    - Desired AR model order (must be one or greater)
C     IQ    - Desired MA model order (must be one or greater)
C     L     - AR model order for large order AR model
C     EPS   - Small positive number to test for ill-conditioning
C             of matrix, required in CHOLESKY used to solve
C             linear equations
C     NEXP  - 2**NEXP is the FFT length used to approximate 
C             integral in computation of white noise variance 
C 
C   Output Parameters: 
C 
C     A     - Complex vector with first p locations containing
C             AR parameter estimates arranged as A(1) to A(IP)
C             (see Notes below)
C     B     - Complex vector of dimension IQx1 of MA parameter 
C             estimates arranged as B(1) to B(IQ)
C     SIG2  - Excitation white noise variance estimate
C     IFLAG - This is a flag indicating condition of termination.
C               =-1, if one of di's in CHOLESKY is less than EPS,
C                    causing premature termination
C               = 0, normal termination
C               = 1, MA filter estimate not minimum-phase causing
C                    premature termination
C          
C   External Subroutines: 
C 
C     FFT         - see Appendix 2A
C     CHOLESKY    - see Appendix 2B
C     CORRELATION - see Appendix 4D
C     LEVINSON    - see Appendix 6B
C     STEPDOWN    - see Appendix 6C
C     AUTOCORR    - see Appendix 7B
C
C   Notes: 
C
C     The calling program must dimension arrays X,A,B.
C     X and A must be dimensioned greater than or equal to
C     2**NEXP and L, respectively.  The arrays U,Y,Z,YZ,BC,P 
C     must be dimensioned greater than or equal to the variable 
C     dimensions shown.  The arrays HH,THETA in LS must be
C     dimensioned greater than or equal to (IP+IQ)x(IP+IQ+1)/2
C     and IP+IQ, respectively.  The arrays AA,BB in VARM must 
C     each be dimensioned greater than or equal to 2**NEXP.  
C     The arrays XL,Y,D in CHOLESKY must be dimensioned
C     greater than or equal to (IP+IQ)x(IP+IQ), IP+IQ, and
C     IP+IQ, respectively.  The arrays R,AA,RHO in AUTOCORR must be
C     dimensioned greater than or equal to L+1,LxL, and L, respectively.
C     The array AA in LEVINSON must be dimensioned to agree with
C     dimensions of AA in AUTOCORR.  The array AA in STEPDOWN must
C     be dimensioned to agree with the dimensions of BC in MAYNEFIR.
C     The array Y in FFT must be dimensioned to be greater than or
C     equal to 2**NEXP.
C
C   Verification Test Case:
C   
C     If the complex test data listed in Appendix 1.A is input
C     as X, with N=32, IP=7, IQ=3, L=21, EPS=1.0E-15, NEXP=9,
C     then the output should be
C       A(1) = ( 0.31355    ,-0.11364    )
C       A(2) = (-1.09737    ,-0.17999    )
C       A(3) = (-0.58869    , 9.76058E-02)
C       A(4) = ( 0.59138    , 0.22864    )
C       A(5) = ( 0.24318    ,-1.20419E-02)
C       A(6) = ( 5.27812E-02,-0.15302    )
C       A(7) = ( 0.14779    ,-3.40991E-02)
C       B(1) = ( 0.25299    ,-7.42263E-02)    
C       B(2) = (-0.44655    ,-9.62517E-02)
C       B(3) = (-7.72834-03, 1.32956E-02)
C       SIG2 = 1.91462 
C       IFLAG=0
C     for a DEC VAX 11/780.
C
      COMPLEX X(1),A(1),B(1),U(N)
      COMPLEX Y(N),Z(N),YZ(N),BC(IQ,IQ)
      DIMENSION P(IQ)
      IFLAG=0
C   Apply autocorrelation method to data, x[n] (step 1)
      CALL AUTOCORR(X,N,L,A,SIG2)
C   Filter data x[n] with estimated A(z) to obtain estimate of
C   input white noise sequence, u[n] (step 2 - (10.23))
      CALL CCZEROFLT(X,N,A,L,U) 
C   Perform second least squares solution to estimate AR and
C   MA parameters as per step 3
      CALL LS(X,U,X,N,IP,IQ,EPS,A,B,IFLAG)
C   Terminate program if equations to be solved are ill-conditioned
      IF(IFLAG.EQ.-1)RETURN
C   Check to make sure that MA filter, 1/B(z), is minimum-phase
C   by checking magnitudes of reflection coefficients and if 
C   not abort
      IF(IQ.EQ.1)BC(1,1)=B(1)
      PMA=1.
      IF(IQ.GT.1)CALL STEPDOWN(IQ,B,PMA,BC,P0,P)
      DO 15 I=1,IQ
      IF(CABS(BC(I,I)).LT.1.0)GO TO 15
      IFLAG=1
      RETURN
15    CONTINUE
C   Filter data, x[n], with estimated 1/B(z) (10.24)
      CALL CPOLEFILT(X,N,B,IQ,Y)
C   Filter estimated input sequence with 1/B(z) (10.25)
      CALL CPOLEFILT(U,N,B,IQ,Z) 
C   Perform last least squares solution for final ARMA
C   parameter estimates (step 5 - (10.27))
      DO 20 I=1,N
20    YZ(I)=Y(I)-Z(I)
      CALL LS(Y,Z,YZ,N,IP,IQ,EPS,A,B,IFLAG) 
C   Terminate program if equations are ill-conditioned
      IF(IFLAG.EQ.-1)RETURN 
      CALL VARM(X,N,A,IP,B,IQ,NEXP,SIG2) 
      RETURN 
      END 
      SUBROUTINE CPOLEFILT(X,N,B,IQ,Y)  
C   This program implements an all-pole filter, 1/B(z), with
C   the initial conditions of the filter set equal to zero.
      COMPLEX X(1),Y(1),B(1)  
      Y(1)=X(1)  
      IF(IQ.EQ.1)GO TO 25  
      DO 20 I=2,IQ  
      Y(I)=X(I)  
      DO 10 J=1,I-1  
10    Y(I)=Y(I)-B(J)*Y(I-J)  
20    CONTINUE  
25    DO 40 I=IQ+1,N  
      Y(I)=X(I)  
      DO 30 J=1,IQ  
30    Y(I)=Y(I)-B(J)*Y(I-J)  
40    CONTINUE  
      RETURN  
      END  
      SUBROUTINE CCZEROFLT(X,N,A,IP,Y)  
C   This program implements an FIR filter with coefficients
C   given by A.  The values of the input X(I) for I < 1 are
C   set equal to zero.
      COMPLEX X(1),Y(1),A(1)  
      Y(1)=X(1)  
      IF(IP.EQ.1)GO TO 25  
      DO 20 I=2,IP  
      Y(I)=X(I)  
      DO 10 J=1,I-1  
10    Y(I)=Y(I)+A(J)*X(I-J)  
20    CONTINUE  
25    DO 40 I=IP+1,N  
      Y(I)=X(I)  
      DO 30 J=1,IP  
30    Y(I)=Y(I)+A(J)*X(I-J)  
40    CONTINUE  
      RETURN  
      END  
      SUBROUTINE LS(Y,Z,YZ,N,IP,IQ,EPS,A,B,IFLAG) 
C   This program sets up the matrices and vectors needed
C   to compute a least squares solution and solves them 
C   using a Cholesky decomposition. 
      COMPLEX Y(1),Z(1),A(1),B(1),HKI,HKJ
      COMPLEX HH((IP+IQ)*(IP+IQ+1)/2),THETA(IP+IQ),YZ(1)
      IPQ=IP+IQ 
      L=1 
      DO 30 J=1,IP+IQ 
      DO 20 I=1,J 
      HH(L)=(0.,0.) 
      DO 10 K=1,N 
      IF(I.LE.IP.AND.K-I.GE.1)HKI=Y(K-I) 
      IF(I.LE.IP.AND.K-I.LE.0)HKI=(0.,0.) 
      IF(I.GT.IP.AND.K-(I-IP).GE.1)HKI=Z(K-(I-IP)) 
      IF(I.GT.IP.AND.K-(I-IP).LE.0)HKI=(0.,0.) 
      IF(J.LE.IP.AND.K-J.GE.1)HKJ=Y(K-J) 
      IF(J.LE.IP.AND.K-J.LE.0)HKJ=(0.,0.) 
      IF(J.GT.IP.AND.K-(J-IP).GE.1)HKJ=Z(K-(J-IP)) 
      IF(J.GT.IP.AND.K-(J-IP).LE.0)HKJ=(0.,0.) 
10    HH(L)=HH(L)+CONJG(HKI)*HKJ
      L=L+1 
20    CONTINUE 
30    CONTINUE 
      DO 50 I=1,IP+IQ 
      THETA(I)=(0.,0.)
      DO 40 K=1,N 
      IF(I.LE.IP.AND.K-I.GE.1)HKI=Y(K-I) 
      IF(I.LE.IP.AND.K-I.LE.0)HKI=(0.,0.) 
      IF(I.GT.IP.AND.K-(I-IP).GE.1)HKI=Z(K-(I-IP)) 
      IF(I.GT.IP.AND.K-(I-IP).LE.0)HKI=(0.,0.) 
40    THETA(I)=THETA(I)+CONJG(HKI)*YZ(K) 
50    CONTINUE
      CALL CHOLESKY(HH,THETA,IPQ,EPS,IFLAG) 
      DO 60 I=1,IP+IQ 
      IF(I.LE.IP)A(I)=-THETA(I) 
60    IF(I.GT.IP)B(I-IP)=THETA(I) 
      RETURN 
      END 
      SUBROUTINE VARM(X,N,A,IP,B,IQ,NEXP,SIG2) 
C   This program computes the estimate of the white
C   noise variance. 
      COMPLEX X(1),A(1),B(1),AA(2**NEXP),BB(2**NEXP) 
      M=2**NEXP 
      AA(1)=(1.,0.) 
      BB(1)=(1.,0.) 
      DO 10 I=2,M 
      IF(I.GT.N)X(I)=(0.,0.) 
      IF(I.LE.IP+1)AA(I)=A(I-1) 
      IF(I.GT.IP+1)AA(I)=(0.,0.) 
      IF(I.LE.IQ+1)BB(I)=B(I-1) 
10    IF(I.GT.IQ+1)BB(I)=(0.,0.) 
      INVRS=1
      CALL FFT(X,NEXP,INVRS)
      CALL FFT(AA,NEXP,INVRS)
      CALL FFT(BB,NEXP,INVRS)
      SIG2=0. 
      DO 20 I=1,M 
20    SIG2=SIG2+(CABS(X(I))**2)*(CABS(AA(I))**2)/(CABS(BB(I))**2) 
      SIG2=SIG2/(N*M) 
      RETURN 
      END 

