      SUBROUTINE MVSE(X,N,IP,L,EPS,MODE,PMV,IFLAG)  
C 
C   This program implements the minimum variance spectral estimator 
C   (MVSE) (11.17) using the modified covariance method estimator of  
C   the autocorrelation matrix (11.27). 
C 
C   Input Parameters: 
C 
C     X     - Complex array of dimension Nx1 of data points 
C     N     - Number of complex data points 
C     IP    - Dimension of autocorrelation matrix used in estimator 
C     L     - Number of samples of spectral estimate desired 
C     EPS   - Small positive number to test for ill-conditioning 
C             of autocorrelation matrix to be inverted, required 
C             in CHOLESKY to solve linear equations 
C     MODE  - If MODE=0, then numerator used in definition of MVSE 
C             is one; if MODE=1, then numerator is given by IP. 
C     
C   Output Parameters: 
C 
C     PMV   - Real array of dimension Lx1 of samples of the MVSE, where 
C             PMV(I) corresponds to the spectral estimate at the  
C             frequency F=-1/2+(I-1)/L 
C     IFLAG - Ill-conditioning indicator for solution of linear equations 
C               =  0 for normal termination 
C               = -1 if one of the di's in Cholesky decomposition 
C                    is less than EPS causing premature termination 
C 
C   External Subroutines: 
C                       
C    CHOLESKY - see Appendix 2B 
C 
C   Notes:  
C 
C     The calling program must dimension arrays X,PMV.  The 
C     arrays B,E,RMAT,A,RINV must be dimensioned greater than or equal 
C     to the variable dimensions shown.  The arrays XL,Y,D in 
C     CHOLESKY must be dimensioned to be greater than or equal 
C     to IPxIP, IP, and IP, respectively. 
C 
C   Verification Test Case: 
C    
C     If the complex test data listed in Appendix 1.A is input 
C     as X, with N=32, IP=11, L=512, EPS=1.0E-15, MODE=1, 
C     then the output should be 
C       PMV(100)=0.04379 
C       PMV(200)=0.18816 
C       PMV(300)=0.47820 
C       PMV(400)=0.03157 
C       PMV(500)=0.30242 
C       IFLAG=0 
C     for a DEC VAX 11/780. 
C 
      COMPLEX X(1),B(IP),E(IP),RMAT(IP*(IP+1)/2),SUM,A(IP*(IP+1)/2),
     * RINV(IP,IP)
      DIMENSION PMV(1)  
      PI2=8.*ATAN(1.)  
C   Compute autocorrelation matrix estimate using modified  
C   covariance method (11.27) 
      CALL CORRMAT(X,N,IP,RMAT)  
      IMAX=IP*(IP+1)/2 
      DO 5 I=1,IMAX 
5     A(I)=RMAT(I) 
C   Begin inversion of autocorrelation matrix
C   Invert autocorrelation matrix by solving IP sets of linear 
C   equations for X or Rxx X = I, where I is the IPxIP identity matrix
      DO 30 J=1,IP
      DO 10 I=1,IP
      B(I)=(0.,0.)
10    IF(I.EQ.J)B(I)=(1.,0.)
      CALL CHOLESKY(A,B,IP,EPS,IFLAG)  
C   Terminate program if matrix ill-conditioned 
      IF(IFLAG.EQ.-1)RETURN 
      DO 20 I=1,IP
20    RINV(I,J)=B(I)
30    CONTINUE
C   Computation of hermitian form  
      DO 70 K=1,L  
      F=-0.5+(K-1.)/L  
      DO 40 J=1,IP  
40    E(J)=CEXP(CMPLX(0.,PI2*(J-1.)*F))  
      SUM=(0.,0.)      
      DO 60 I=1,IP  
      DO 50 J=1,IP
      SUM=SUM+CONJG(E(I))*RINV(I,J)*E(J)
50    CONTINUE
60    CONTINUE
C   Compute PSD values (11.17) 
      IF(MODE.EQ.0)PMV(K)=REAL(1./SUM)  
      IF(MODE.EQ.1)PMV(K)=REAL(IP/SUM)  
70    CONTINUE  
      RETURN  
      END  
      SUBROUTINE CORRMAT(X,N,IP,RMAT)  
C   This program computes an estimate of the IPxIP autocorrelation 
C   matrix by using the modified covariance method. 
      COMPLEX X(1),RMAT(1)  
      L=0 
      DO 30 J=1,IP  
      DO 20 I=1,J  
      L=L+1 
      RMAT(L)=(0.,0.)  
      DO 10 K=IP+1,N  
10    RMAT(L)=RMAT(L)+CONJG(X(K-I))*X(K-J)+X(K-IP+I)*  
     * CONJG(X(K-IP+J))
      RMAT(L)=RMAT(L)/(2.*(N-IP)) 
20    CONTINUE 
30    CONTINUE 
      RETURN 
      END
