      SUBROUTINE MYWE(X,N,IP,IQ,EPS,A,B,SIG2,IFLAG) 
C
C   This program implements the modified Yule-Walker equation 
C   estimator for the AR parameters of an ARMA process (10.7).
C   The unbiased autocorrelation estimator is used.  The equations 
C   are solved using the recursive algorithm of (10.8)-(10.13).
C   The MA parameters and the white noise variance are estimated
C   by filtering the data by an estimate of A(z) and applying
C   Durbin's method (see Section 8.4) to the data at the output
C   of the filter.  The large order AR model in Durbin's method
C   is chosen to have an order of N/5.
C
C   Input Parameters:
C
C     X     - Complex vector of dimension Nx1 of data points
C     N     - Number of complex data points
C     IP    - Desired AR model order (must be one or greater)
C     IQ    - Desired MA model order (must be one or greater)
C     EPS   - A small positive number to test for ill-conditioning
C             of matrix
C    
C   Output Parameters:
C
C     A     - Complex vector of dimension IPx1 of AR parameter
C             estimates arranged as A(1) to A(IP)
C     B     - Complex vector of dimension IQx1 of MA parameter
C             estimates arranged as B(1) to B(IQ)
C     SIG2  - Excitation white noise variance estimate
C     IFLAG - Ill-conditioning indicator for solution of 
C             modified Yule-Walker equations
C               =  0 for normal termination
C               = -1 if magnitude of one of complex rho's is 
C                    less than EPS causing premature termination
C       
C   External Subroutines:
C
C     CORRELATION - see Appendix 4D
C     LEVINSON    - see Appendix 6B
C     AUTOCORR    - see Appendix 7B
C     DURBIN      - see Appendix 8A
C
C   Notes:
C
C     The calling program must dimension arrays X,A,B.  
C     The arrays R,Y must be dimensioned greater 
C     than or equal to the variable dimensions shown.
C     The arrays AA,BB in TOEPEQN must be dimensioned
C     to both be greater than or equal to IPxIP.  The arrays
C     A,AP in DURBIN must be dimensioned greater than or equal
C     to [N/5] and [N/5]+1, respectively.  ([M] denotes the
C     largest integer less than or equal to M).  The arrays 
C     R,AA,RHO in AUTOCORR must be dimensioned greater than or 
C     equal to [N/5]+1, [N/5]x[N/5], and [N/5], respectively.  
C     The array AA in LEVINSON must be dimensioned to agree with 
C     the dimensions of AA in AUTOCORR.
C
C   Verification Test Case:
C   
C     If the complex test data listed in Appendix 1.A is input
C     as X, with N=32, IP=7, IQ=3, EPS=1.0E-15, then the output 
C     should be
C       A(1) = ( 1.66754    ,-0.13065    )
C       A(2) = (-2.26033    ,-0.25969    )
C       A(3) = (-0.71092    ,-6.01127E-02)
C       A(4) = ( 0.96275    , 0.41967    )
C       A(5) = ( 0.25055    , 0.22180    )
C       A(6) = (-0.90021    ,-0.24728    )
C       A(7) = ( 1.43252    ,-0.21511    )
C       B(1) = (-0.49970    ,-0.10093    )    
C       B(2) = (-3.66813E-02,-7.14685E-02)
C       B(3) = ( 0.16940    , 4.09755E-02)
C       SIG2 = 9.21820 
C       IFLAG=0
C     for a DEC VAX 11/780.
C
      COMPLEX X(1),A(1),B(1),R(IP+IQ+1),Y(N)
C   Compute unbiased autocorrelation estimates
      LAG=IP+IQ+1 
      CALL CORRELATION(N,LAG,0,X,X,R) 
C   Solve modified Yule-Walker equations using recursive
C   algorithm of (10.8)-(10.13)
      CALL TOEPEQN(R,IP,IQ,EPS,A,IFLAG)
      IF(IFLAG.EQ.-1)RETURN 
C   Filter data by estimated A(z) to yield approximate
C   MA time series (set initial conditions equal to zero)
20    CALL CZEROFLT(X,N,IP,A,Y) 
C   Apply Durbin's method with large order AR model order of
C   N/5 to estimate the MA parameters and white noise variance
      L=N/5 
      CALL DURBIN(Y,N,IQ,L,B,SIG2) 
      RETURN 
      END 
      SUBROUTINE CZEROFLT(X,N,IP,A,Y) 
C   This program implements an FIR filter with coefficients
C   given by A.  The values of the input X(I) for I < 1 are
C   set equal to zero.
      COMPLEX X(1),Y(1),A(1)
      Y(1)=X(1) 
      IF(IP.EQ.1)GO TO 25 
      DO 20 I=2,IP 
      Y(I)=X(I) 
      DO 10 J=1,I-1 
10    Y(I)=Y(I)+A(J)*X(I-J) 
20    CONTINUE 
25    DO 40 I=IP+1,N 
      Y(I)=X(I) 
      DO 30 J=1,IP 
30    Y(I)=Y(I)+A(J)*X(I-J)
40    CONTINUE
      RETURN
      END
      SUBROUTINE TOEPEQN(R,IP,IQ,EPS,A,IFLAG)
C  This program solves the modified Yule-Walker
C  equations using a generalized Levinson recursion.
      COMPLEX R(1),A(1),AA(IP,IP),BB(IP,IP),C,D,RHO,CORR
      IFLAG=0
C  Initialize recursion (10.8)
      AA(1,1)=-R(IQ+2)/R(IQ+1)
      IF(IP.EQ.1)GO TO 60
      BB(1,1)=-R(IQ)/R(IQ+1)
      RHO=(1.-AA(1,1)*BB(1,1))*R(IQ+1)
C  Check to see if 2x2 matrix is singular
      IF(CABS(RHO).GT.EPS)GO TO 5
      IFLAG=-1
      RETURN
C  Start main recursion
5     DO 50 K=2,IP
C  Compute ak[k] (10.9)
      C=-R(IQ+1+K)
      DO 10 L=1,K-1
      LAG=IQ+K-L
      IF(LAG.GE.0)CORR=R(LAG+1)
      IF(LAG.LT.0)CORR=CONJG(R(-LAG+1))
10    C=C-AA(L,K-1)*CORR
      AA(K,K)=C/RHO
C  Update in order the ak[i]'s (10.10)
      DO 20 I=1,K-1
20    AA(I,K)=AA(I,K-1)+AA(K,K)*BB(K-I,K-1)
      IF(K.EQ.IP)GO TO 60
C  Compute bk[k] (10.11)
      LAG=IQ-K
      IF(LAG.GE.0)D=-R(LAG+1)
      IF(LAG.LT.0)D=-CONJG(R(-LAG+1))
      DO 30 L=1,K-1
      LAG=IQ-K+L
      IF(LAG.GE.0)CORR=R(LAG+1)
      IF(LAG.LT.0)CORR=CONJG(R(-LAG+1))
30    D=D-BB(L,K-1)*CORR
      BB(K,K)=D/RHO
C  Update in order the bk[i]'s (10.12)
      DO 40 I=1,K-1
40    BB(I,K)=BB(I,K-1)+BB(K,K)*AA(K-I,K-1)
C  Check to see if (k+1)x(k+1) matrix is singular (10.14)
      RHO=(1.-AA(K,K)*BB(K,K))*RHO
      IF(CABS(RHO).GT.EPS)GO TO 50
      IFLAG=-1
      RETURN
50    CONTINUE
C  Find final solution
60    DO 70 I=1,IP
70    A(I)=AA(I,IP)
      RETURN
      END







