      SUBROUTINE PERIODOGRAM(X,N,W,NEXP,PPER)
C
C   This program computes a data windowed periodogram.  For
C   the case where no data windowing is used the periodogram
C   is given by (4.2).  The periodogram is evaluated at the 
C   frequencies F=-1/2+(I-1)/L where I=1,2,...,L. The number 
C   of frequencies is given by L=2**NEXP.
C
C   Input Parameters:
C
C     X    - Complex array of dimension 2**NEXPx1 with first
C            N locations containing the data points
C     N    - Number of data points
C     W    - Real array of dimension Nx1 of data window weights;
C            the weights W(1),W(2),...,W(N) should be
C            symmetric about the point (N+1)/2.  If no windowing 
C            is desired, then set W(I)=1 for I=1,2,...,N.
C     NEXP - Power of two which determines number of frequency
C            samples desired; L=2**NEXP must be chosen so that
C            L is greater than or equal to N
C
C   Output Parameters:
C
C     PPER - Real array of dimension L=2**NEXPx1 of samples 
C            of the periodogram, where PPER(I) corresponds to
C            the spectral estimate at the frequency F=-1/2+(I-1)/L
C
C   External Subroutines:
C
C     FFT    - see Appendix 2A
C
C   Notes: 
C     
C     The calling program must dimension the arrays X,W,PPER.
C     The array Y in FFT must be dimensioned greater than or
C     equal to 2**NEXP.
C         
C   Verification Test Case: 
C
C     If the complex data test case listed in Appendix 1.A is 
C     input as X, with N=32, W(I)=1 for I=1,2,...,32 and NEXP=9,
C     then the output should be
C       PPER(100) = 0.89216
C       PPER(200) = 0.15918
C       PPER(300) = 0.76762
C       PPER(400) = 0.76319
C       PPER(500) = 1.35960
C     for a DEC VAX 11/780.
C
      COMPLEX X(1)
      DIMENSION W(1),PPER(1)
C   Window the data
      DO 10 I=1,N
10    X(I)=X(I)*W(I)
C   Compute FFT of windowed data
      L=2**NEXP
      IF(L.EQ.N)GO TO 25
      DO 20 I=N+1,L
20    X(I)=(0.,0.)
25    INVRS=1
      CALL FFT(X,NEXP,INVRS)
C   Compute samples of periodogram and transpose halves of
C   FFT output so that first PSD sample is at a frequency of -1/2
      DO 30 I=1,L/2
      PPER(I+L/2)=(CABS(X(I))**2)/N
30    PPER(I)=(CABS(X(I+L/2))**2)/N
      RETURN
      END

