      SUBROUTINE RMLE(X,N,IP,A,SIG2) 
C   This program implements the recursive maximum likelihood
C   estimator for the AR parameters (7.46)-(7.52).  The data 
C   must be real.
C
C   Input Parameters:
C
C     X    - Real array of dimension Nx1 of data points
C     N    - Number of data points
C     IP   - AR model order desired
C
C   Output Parameters:
C
C     A    - Real array of dimension IPx1 of AR filter
C            parameter estimates arranged as A(1) to A(IP)
C     SIG2 - Excitation white noise variance estimate
C
C   Notes: 
C
C     The calling program must dimension the X,A arrays.
C     The arrays S,AA,RHO in the main subroutine RMLE 
C     and the arrays AP,B in the called subroutine COEFF
C     must be dimensioned greater than or equal to the variable 
C     dimensions shown.  The arrays S,AA in the called
C     subroutines COEFF, ARUPT, and SMAT must be dimensioned
C     to agree with dimensions of S,AA in the main subroutine
C     RMLE.
C          
C   Verification Test Case:
C
C     If the real test data listed in Appendix 1.A is input
C     as X, with N=32, and IP=10, then the output should be
C       A(1) = -7.64353E-02
C       A(2) = -1.26934
C       A(3) = -0.30781
C       A(4) =  0.85033
C       A(5) = -0.49144
C       A(6) =  0.36946
C       A(7) =  0.75698
C       A(8) = -0.18951
C       A(9) = -0.62877
C       A(10)=  0.21280
C       SIG2 =  0.21543
C     for a DEC VAX 11/780.
C  
      DIMENSION X(1),S(IP+1,IP+1),AA(IP,IP),A(1),RHO(IP),ROOT(3) 
C   Compute elements of S matrix (7.52)
      CALL SMAT(X,N,IP,S) 
C   Begin recursion
      DO 10 I=1,IP 
C   Obtain coefficients of cubic equation to be solved
C   for next reflection coefficient (7.48),(7.51)
      CALL COEFF(I,IP,S,N,AA,P,Q,R,C,D,E) 
C   Solve cubic equation (7.49)
      CALL CUBIC(P,Q,R,NREAL,ROOT) 
C   If there is only one real root, then set reflection
C   coefficient equal to it.
      AA(I,I)=ROOT(1) 
C   If there are three real roots, find the one which is 
C   within the [-1,1] interval and which maximizes the
C   likelihood function. (7.50)
      IF(NREAL.EQ.3)CALL TEST(ROOT,C,D,E,I,N,AA(I,I)) 
C   Update the likelihood function (7.51)
      CALL ARUPT(AA,I,S,N,C,D,E,RHO(I)) 
10    CONTINUE 
C   Obtain final AR parameter estimates
      DO 20 I=1,IP
20    A(I)=AA(I,IP)
      SIG2=RHO(IP)
      RETURN 
      END 
      SUBROUTINE COEFF(I,IP,S,N,AA,P,Q,R,C,D,E) 
C   This program computes the coefficients of the cubic
C   equation to be solved for the reflection coefficient.
      DIMENSION S(IP+1,IP+1),B(IP),AA(IP,IP),AP(IP) 
      IF(I.EQ.1)GO TO 60 
      I1=I-1 
      I2=I+1 
      AP(1)=1. 
      DO 20 J=1,I1 
20    AP(J+1)=AA(J,I1)
      DO 35 J=1,I
35    B(J)=AP(I+1-J)
      C=0. 
      D=0. 
      DO 40 L=1,I 
      DO 30 K=1,I 
      C=C+AP(K)*B(L)*S(K,L+1) 
      D=D+B(K)*B(L)*S(K+1,L+1) 
30    CONTINUE 
40    CONTINUE 
      GO TO 70 
60    C=S(1,2) 
      D=S(2,2) 
      E=S(1,1) 
70    P=((N-2.*I)/(N-I))*(C/D) 
      Q=(I*E+N*D)/(D*(I-N)) 
      R=N*C/(D*(I-N)) 
      RETURN 
      END 
      SUBROUTINE ARUPT(AA,I,S,N,C,D,E,POW) 
C   This program updates in order the prediction coefficients
C   and the prediction error power.
      DIMENSION AA(IP,IP),S(IP+1,IP+1) 
      IF(I.EQ.1)GO TO 15 
      I1=I-1 
      DO 10 J=1,I1 
10    AA(J,I)=AA(J,I1)+AA(I,I)*AA(I-J,I1) 
15    E=E+2.*C*AA(I,I)+D*AA(I,I)**2
      POW=E/FLOAT(N) 
      RETURN 
      END 
      SUBROUTINE SMAT(X,N,IP,S) 
C   This program computes the S data matrix required in the
C   likelihood function.
      DIMENSION S(IP+1,IP+1),X(1) 
      IP1=IP+1 
      DO 30 J=1,IP1 
      DO 20 I=1,IP1 
      S(I,J)=0. 
      DO 10 K=1,N-(I-1)-(J-1) 
10    S(I,J)=S(I,J)+X(K+I-1)*X(K+J-1) 
20    CONTINUE 
30    CONTINUE 
      RETURN 
      END 
      SUBROUTINE CUBIC(P,Q,R,NREAL,ROOT) 
C   This program determines the real roots of a cubic equation.
      DIMENSION ROOT(3) 
      PI=4.*ATAN(1.) 
      A=(3.*Q-P*P)/3. 
      B=(2.*P**3-9.*P*Q+27.*R)/27. 
      D=B*B/4.+A**3/27. 
      IF(D.GT.0.)NREAL=1 
      IF(D.LE.0.)NREAL=3 
      IF(NREAL.EQ.3)GO TO 10 
      ARG1=-B/2.+SQRT(D) 
      SARG1=ARG1/ABS(ARG1) 
      ARG2=-B/2.-SQRT(D) 
      SARG2=ARG2/ABS(ARG2) 
      AA=SARG1*((ABS(ARG1))**(1./3.)) 
      BB=SARG2*((ABS(ARG2))**(1./3.)) 
      ROOT(1)=AA+BB-P/3. 
      GO TO 20 
10    C=2.*SQRT(-A/3.) 
      Z=(3.*B)/(A*C) 
      ALPHA=ATAN((SQRT(1.-Z*Z))/ABS(Z)) 
      IF(Z.LT.0)ALPHA=PI-ALPHA 
      Y=(1./3.)*ALPHA 
      ROOT(1)=C*COS(Y)-P/3. 
      ROOT(2)=C*COS(Y+2.*PI/3.)-P/3. 
      ROOT(3)=C*COS(Y+4.*PI/3.)-P/3. 
20    RETURN 
      END 
      SUBROUTINE TEST(ROOT,C,D,E,I,N,XKEST) 
C   This program checks the three real roots of a cubic
C   equation to determine which one is the maximum likelihood
C   estimate of the reflection coefficient.
      DIMENSION ROOT(3) 
      FMAX=-10000000. 
      DO 30 J=1,3 
      IF(ROOT(J).LT.-1..OR.ROOT(J).GT.1.)GO TO 30 
      Q=E+2.*ROOT(J)*C+ROOT(J)*ROOT(J)*D 
      F=(I/2.)*ALOG10(1.-ROOT(J)*ROOT(J))-(N/2.)*ALOG10(Q/N)
      IF(F.GT.FMAX)XKEST=ROOT(J)
      IF(F.GT.FMAX)FMAX=F
30    CONTINUE
      RETURN
      END

